@@ -1,21 +1,20 @@
 /*
- * wavemon - a wireless network monitoring aplication
+ * wavemon - a wireless network monitoring application
  *
  * Copyright (c) 2001-2002 Jan Morgenstern <jan@jm-music.de>
  *
- * wavemon is free software; you can redistribute it and/or modify it under
- * the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2, or (at your option) any later
- * version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- * wavemon is distributed in the hope that it will be useful, but WITHOUT ANY
- * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
- * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
- * details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with wavemon; see the file COPYING.  If not, write to the Free Software
- * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 #include "wavemon.h"
 #include <pwd.h>
@@ -47,6 +46,13 @@ static char *sort_order[] = {
 	NULL
 };
 
+static char *scan_filter_bands[] = {
+	[SCAN_FILTER_BAND_BOTH]	= "Both",
+	[SCAN_FILTER_BAND_2G]	= "2.4GHz",
+	[SCAN_FILTER_BAND_5G]	= "5GHz",
+	NULL
+};
+
 static char *screen_names[] = {
 	[SCR_INFO]	= "Info screen",
 	[SCR_LHIST]	= "Histogram",
@@ -74,6 +80,8 @@ struct wavemon_conf conf = {
 
 	.scan_sort_order	= SO_CHAN_SIG,
 	.scan_sort_asc		= false,
+	.scan_hidden_essids	= true,
+	.scan_filter_band	= SCAN_FILTER_BAND_BOTH,
 	.lthreshold_action	= TA_DISABLED,
 	.lthreshold		= -80,
 	.hthreshold_action	= TA_DISABLED,
@@ -357,6 +365,22 @@ static void init_conf_items(void)
 	item->list	= on_off_names;
 	ll_push(conf_items, "*", item);
 
+	item = calloc(1, sizeof(*item));
+	item->name	= strdup("Scan band selection");
+	item->cfname	= strdup("scan_filter_band");
+	item->type	= t_list;
+	item->v.i	= &conf.scan_filter_band;
+	item->list	= scan_filter_bands;
+	ll_push(conf_items, "*", item);
+
+	item = calloc(1, sizeof(*item));
+	item->name	= strdup("Scan hidden ESSIDs");
+	item->cfname	= strdup("scan_hidden_essids");
+	item->type	= t_list;
+	item->v.i	= &conf.scan_hidden_essids;
+	item->list	= on_off_names;
+	ll_push(conf_items, "*", item);
+
 	item = calloc(1, sizeof(*item));
 	item->type = t_sep;
 	ll_push(conf_items, "*", item);
