@@ -1,21 +1,20 @@
 /*
- * wavemon - a wireless network monitoring aplication
+ * wavemon - a wireless network monitoring application
  *
  * Copyright (c) 2001-2002 Jan Morgenstern <jan@jm-music.de>
  *
- * wavemon is free software; you can redistribute it and/or modify it under
- * the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2, or (at your option) any later
- * version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- * wavemon is distributed in the hope that it will be useful, but WITHOUT ANY
- * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
- * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
- * details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with wavemon; see the file COPYING.  If not, write to the Free Software
- * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 #include "iw_if.h"
 #include "iw_nl80211.h"
@@ -48,14 +47,9 @@ static struct iw_extrema {
 	float	max;
 } e_signal, e_noise, e_snr;
 
-static void init_extrema(struct iw_extrema *ie)
-{
-	memset(ie, 0, sizeof(*ie));
-}
-
 static void track_extrema(const float new_sample, struct iw_extrema *ie)
 {
-	if (! ie->initialised) {
+	if (!ie->initialised) {
 		ie->initialised = true;
 		ie->min = ie->max = new_sample;
 	} else if (new_sample < ie->min) {
@@ -69,13 +63,13 @@ static char *fmt_extrema(const struct iw_extrema *ie, const char *unit)
 {
 	static char range[256];
 
-	if (! ie->initialised)
+	if (!ie->initialised)
 		snprintf(range, sizeof(range), "unknown");
 	else if (ie->min == ie->max)
 		snprintf(range, sizeof(range), "%+.0f %s", ie->min, unit);
 	else
-		snprintf(range, sizeof(range), "%+.0f..%+.0f %s", ie->min,
-								ie->max, unit);
+		snprintf(range, sizeof(range), "%+.0f %s ... %+.0f %s",
+				ie->min, unit, ie->max, unit);
 	return range;
 }
 
@@ -95,7 +89,7 @@ static char *fmt_extrema(const struct iw_extrema *ie, const char *unit)
 #define IW_STACKSIZE		1024
 static struct iw_levelstat	iw_stats_cache[IW_STACKSIZE];
 static uint32_t			count;
-#define COUNTMAX		(typeof(count))-1
+const static uint32_t		COUNTMAX = ~0;
 
 static void iw_cache_insert(const struct iw_levelstat new)
 {
@@ -242,7 +236,7 @@ static void display_lhist(void)
 		iwl = iw_cache_get(x);
 
 		/* Clear screen and set up horizontal grid lines */
-		wattrset(w_lhist, COLOR_PAIR(CP_STATBKG));
+		wattrset(w_lhist, COLOR_PAIR(CP_BLUE));
 		for (y = 1; y <= HIST_MAXYLEN; y++)
 			mvwaddch(w_lhist, hist_y(y), hist_x(x), (y % 5) ? ' ' : '-');
 
@@ -258,14 +252,14 @@ static void display_lhist(void)
 					       conf.sig_min - conf.noise_max,
 					       conf.sig_max - conf.noise_min);
 
-			wattrset(w_lhist, COLOR_PAIR(CP_STATSNR));
+			wattrset(w_lhist, COLOR_PAIR(CP_BLUE_ON_BLUE));
 			for (y = 1; y <= clamp(snr_level, 1, HIST_MAXYLEN); y++)
 				mvwaddch(w_lhist, hist_y(y), hist_x(x), ' ');
 		}
 
 		if (! (iwl.flags & IW_QUAL_NOISE_INVALID)) {
 			noise_level = hist_level(iwl.noise, conf.noise_min, conf.noise_max);
-			plot_colour = noise_level > snr_level ? CP_STATNOISE : CP_STATNOISE_S;
+			plot_colour = noise_level > snr_level ? CP_RED : CP_RED_ON_BLUE;
 			hist_plot(noise_level, x, plot_colour);
 
 		} else if (x == LEVEL_TAG_POS && ! (iwl.flags & IW_QUAL_LEVEL_INVALID)) {
@@ -276,7 +270,7 @@ static void display_lhist(void)
 			 * This is only supported for signal levels, when the screen is not
 			 * shared by several graphs (each having a different scale).
 			 */
-			wattrset(w_lhist, COLOR_PAIR(CP_STATSIG));
+			wattrset(w_lhist, COLOR_PAIR(CP_GREEN));
 			for (y = 1; y <= HIST_MAXYLEN; y++) {
 				if (y != 1 && (y % 5) && y != HIST_MAXYLEN)
 					continue;
@@ -289,7 +283,7 @@ static void display_lhist(void)
 
 		if (! (iwl.flags & IW_QUAL_LEVEL_INVALID)) {
 			sig_level   = hist_level(iwl.signal, conf.sig_min, conf.sig_max);
-			plot_colour = sig_level > snr_level ? CP_STATSIG : CP_STATSIG_S;
+			plot_colour = sig_level > snr_level ? CP_GREEN : CP_GREEN_ON_BLUE;
 			hist_plot(sig_level, x, plot_colour);
 		}
 	}
@@ -299,32 +293,37 @@ static void display_lhist(void)
 
 static void display_key(WINDOW *w_key)
 {
+	char buf[128];
 	/* Clear the (one-line) screen) */
 	wmove(w_key, 1, 1);
 	wclrtoborder(w_key);
 
 	wattrset(w_key, COLOR_PAIR(CP_STANDARD));
 	waddch(w_key, '[');
-	wattrset(w_key, COLOR_PAIR(CP_STATSIG));
+	wattrset(w_key, COLOR_PAIR(CP_GREEN));
 	waddch(w_key, ACS_HLINE);
 	wattrset(w_key, COLOR_PAIR(CP_STANDARD));
 
 	if (e_noise.initialised && e_snr.initialised) {
-		wprintw(w_key, "] sig lvl (%s)  [", fmt_extrema(&e_signal, "dBm"));
+		snprintf(buf, sizeof(buf), "] sig lvl (%s)  [", fmt_extrema(&e_signal, "dBm"));
+		waddstr(w_key, buf);
 
-		wattrset(w_key, COLOR_PAIR(CP_STATNOISE));
+		wattrset(w_key, COLOR_PAIR(CP_RED));
 		waddch(w_key, ACS_HLINE);
 		wattrset(w_key, COLOR_PAIR(CP_STANDARD));
 
-		wprintw(w_key, "] ns lvl (%s)  [", fmt_extrema(&e_noise, "dBm"));
+		snprintf(buf, sizeof(buf), "] ns lvl (%s)  [", fmt_extrema(&e_noise, "dBm"));
+		waddstr(w_key, buf);
 
-		wattrset(w_key, COLOR_PAIR(CP_STATSNR));
+		wattrset(w_key, COLOR_PAIR(CP_BLUE_ON_BLUE));
 		waddch(w_key, ' ');
 
 		wattrset(w_key, COLOR_PAIR(CP_STANDARD));
-		wprintw(w_key, "] S-N ratio (%s)", fmt_extrema(&e_snr, "dB"));
+		snprintf(buf, sizeof(buf), "] S-N ratio (%s)", fmt_extrema(&e_snr, "dB"));
+		waddstr(w_key, buf);
 	} else {
-		wprintw(w_key, "] signal level (%s)", fmt_extrema(&e_signal, "dBm"));
+		snprintf(buf, sizeof(buf), "] signal level (%s)", fmt_extrema(&e_signal, "dBm"));
+		waddstr(w_key, buf);
 	}
 
 	wrefresh(w_key);
@@ -335,10 +334,7 @@ void scr_lhist_init(void)
 	w_lhist = newwin_title(0, HIST_WIN_HEIGHT, "Level histogram", true);
 	w_key   = newwin_title(HIST_MAXYLEN + 1, KEY_WIN_HEIGHT, "Key", false);
 
-	init_extrema(&e_signal);
-	init_extrema(&e_noise);
-	init_extrema(&e_snr);
-	sampling_init();
+	sampling_init(true);
 
 	display_key(w_key);
 }
