@@ -1,21 +1,20 @@
 /*
- * wavemon - a wireless network monitoring aplication
+ * wavemon - a wireless network monitoring application
  *
  * Copyright (c) 2001-2002 Jan Morgenstern <jan@jm-music.de>
  *
- * wavemon is free software; you can redistribute it and/or modify it under
- * the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2, or (at your option) any later
- * version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- * wavemon is distributed in the hope that it will be useful, but WITHOUT ANY
- * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
- * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
- * details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with wavemon; see the file COPYING.  If not, write to the Free Software
- * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 #include "wavemon.h"
 #include <locale.h>
@@ -26,30 +25,35 @@
 /**
  * screen switching table
  * @key_name:	name under which the screen appears in the menu bar
+ * @shortcut:   1-character shortcut key for @key_name
  * @init:	screen initialisation function pointer
  * @loop:	screen update function pointer (connected to menu)
  * @fini:	screen cleanup function pointer
  */
 static const struct {
 	const char *const	key_name;
+	const char              shortcut;
 	void		 	(*init)(void);
 	int			(*loop)(WINDOW *);
 	void			(*fini)(void);
 } screens[] = {
 	[SCR_INFO]	= {
 		.key_name = "info",
+		.shortcut = 'i',
 		.init	  = scr_info_init,
 		.loop	  = scr_info_loop,
 		.fini	  = scr_info_fini
 	},
 	[SCR_LHIST]	= {
 		.key_name = "lhist",
+		.shortcut = 'l',
 		.init	  = scr_lhist_init,
 		.loop	  = scr_lhist_loop,
 		.fini	  = scr_lhist_fini
 	},
 	[SCR_SCAN]	= {
 		.key_name = "scan",
+		.shortcut = 's',
 		.init	  = scr_aplst_init,
 		.loop	  = scr_aplst_loop,
 		.fini	  = scr_aplst_fini
@@ -65,24 +69,28 @@ static const struct {
 	},
 	[SCR_PREFS]	= {
 		.key_name = "prefs",
+		.shortcut = 'p',
 		.init	  = scr_conf_init,
 		.loop	  = scr_conf_loop,
 		.fini	  = scr_conf_fini
 	},
 	[SCR_HELP]	= {
 		.key_name = "help",
+		.shortcut = 'h',
 		.init	  = scr_help_init,
 		.loop	  = scr_help_loop,
 		.fini	  = scr_help_fini
 	},
 	[SCR_ABOUT]	= {
 		.key_name = "about",
+		.shortcut = 'a',
 		.init	  = scr_about_init,
 		.loop	  = scr_about_loop,
 		.fini	  = scr_about_fini
 	},
 	[SCR_QUIT]	= {
 		.key_name = "quit",
+		.shortcut = 'q',
 	}
 };
 
@@ -103,26 +111,33 @@ static void sig_winch(int signo)
 static WINDOW *init_menubar(const enum wavemon_screen active)
 {
 	WINDOW *menu = newwin(1, WAV_WIDTH, WAV_HEIGHT, 0);
-	enum wavemon_screen cur;
+	char fkey[8];
 
 	nodelay(menu, TRUE);
 	keypad(menu, TRUE);
 	wmove(menu, 0, 0);
-	for (cur = SCR_INFO; cur <= SCR_QUIT; cur++) {
-		wattrset(menu, A_REVERSE | A_BOLD);
-		wprintw(menu, "F%d", cur + 1);
+	for (enum wavemon_screen cur = SCR_INFO; cur <= SCR_QUIT; cur++) {
+		const char *p = screens[cur].key_name;
 
-		wattrset(menu, cur != active ? COLOR_PAIR(CP_INACTIVE)
-					     : COLOR_PAIR(CP_ACTIVE) | A_BOLD);
-		if (*screens[cur].key_name) {
+		if (*p) {
+			snprintf(fkey, sizeof(fkey), "F%d", cur + 1);
+			wattrset(menu, A_REVERSE | A_BOLD);
+			waddstr(menu, fkey);
 
-			wattron(menu, A_UNDERLINE);
-			waddch(menu, screens[cur].key_name[0]);
-			wattroff(menu, A_UNDERLINE);
+			wattrset(menu, cur != active ? COLOR_PAIR(CP_CYAN)
+						     : COLOR_PAIR(CP_CYAN_ON_BLUE) | A_BOLD);
 
-			wprintw(menu, "%-5s", screens[cur].key_name + 1);
-		} else  {
-			wprintw(menu, "%-6s", "");
+			for (int i = 0; i < MAX_MENU_KEY; i++) {
+				if (*p == screens[cur].shortcut)	{
+					wattron(menu, A_UNDERLINE);
+					waddch(menu, *p++);
+					wattroff(menu, A_UNDERLINE);
+				} else if (*p) {
+					waddch(menu, *p++);
+				} else {
+					waddch(menu, ' ');
+				}
+			}
 		}
 	}
 	wrefresh(menu);
@@ -165,28 +180,18 @@ int main(int argc, char *argv[])
 		bg_color = -1;
 		use_default_colors();
 	}
+
 	init_pair(CP_STANDARD,	COLOR_WHITE,	bg_color);
-	init_pair(CP_SCALEHI,	COLOR_RED,	bg_color);
-	init_pair(CP_SCALEMID,	COLOR_YELLOW,	bg_color);
-	init_pair(CP_SCALELOW,	COLOR_GREEN,	bg_color);
-	init_pair(CP_WTITLE,	COLOR_CYAN,	bg_color);
-	init_pair(CP_INACTIVE,	COLOR_CYAN,	bg_color);
-	init_pair(CP_ACTIVE,	COLOR_CYAN,	COLOR_BLUE);
+	init_pair(CP_RED,	COLOR_RED,	bg_color);
+	init_pair(CP_YELLOW,	COLOR_YELLOW,	bg_color);
+	init_pair(CP_GREEN,	COLOR_GREEN,	bg_color);
+	init_pair(CP_CYAN,	COLOR_CYAN,	bg_color);
+	init_pair(CP_BLUE,	COLOR_BLUE,	bg_color);
 
-	init_pair(CP_STATSIG,	  COLOR_GREEN,	bg_color);
-	init_pair(CP_STATNOISE,	  COLOR_RED,	bg_color);
-	init_pair(CP_STATSNR,	  COLOR_BLUE,	COLOR_BLUE);
-	init_pair(CP_STATBKG,	  COLOR_BLUE,	bg_color);
-	init_pair(CP_STATSIG_S,	  COLOR_GREEN,	COLOR_BLUE);
-	init_pair(CP_STATNOISE_S, COLOR_RED,	COLOR_BLUE);
-
-	init_pair(CP_PREF_NORMAL, COLOR_WHITE,	bg_color);
-	init_pair(CP_PREF_SELECT, COLOR_WHITE,	COLOR_BLUE);
-	init_pair(CP_PREF_ARROW,  COLOR_RED,	bg_color);
-
-	init_pair(CP_SCAN_CRYPT,  COLOR_RED,	bg_color);
-	init_pair(CP_SCAN_UNENC,  COLOR_GREEN,	bg_color);
-	init_pair(CP_SCAN_NON_AP, COLOR_YELLOW, bg_color);
+	init_pair(CP_RED_ON_BLUE,	COLOR_RED,	COLOR_BLUE);
+	init_pair(CP_GREEN_ON_BLUE,	COLOR_GREEN,	COLOR_BLUE);
+	init_pair(CP_CYAN_ON_BLUE,	COLOR_CYAN,	COLOR_BLUE);
+	init_pair(CP_BLUE_ON_BLUE,	COLOR_BLUE,	COLOR_BLUE);
 
 	/* Override signal handlers installed during ncurses initialisation. */
 	xsignal(SIGCHLD, SIG_IGN);
@@ -236,41 +241,14 @@ int main(int argc, char *argv[])
 				}
 
 				/* Main menu */
-				switch (key) {
-				case KEY_F(1):
-				case '1':
-				case 'i':
-					next = SCR_INFO;
-					break;
-				case KEY_F(2):
-				case '2':
-				case 'l':
-					next = SCR_LHIST;
-					break;
-				case KEY_F(3):
-				case 's':
-				case '3':
-					next = SCR_SCAN;
-					break;
-				case KEY_F(7):
-				case 'p':
-				case '7':
-					next = SCR_PREFS;
-					break;
-				case KEY_F(8):
-				case 'h':
-				case '8':
-					next = SCR_HELP;
-					break;
-				case KEY_F(9):
-				case 'a':
-				case '9':
-					next = SCR_ABOUT;
-					break;
-				case KEY_F(10):
-				case 'q':
-				case '0':
-					next = SCR_QUIT;
+				for (enum wavemon_screen s = SCR_INFO; s <= SCR_QUIT; s++) {
+					if (*screens[s].key_name && (
+					    key == (s == SCR_QUIT ? '0' : '1' + s) ||
+					    key == KEY_F(s + 1) ||
+					    key == screens[s].shortcut)) {
+						next = s;
+						break;
+					}
 				}
 			} while (next == cur);
 		}
