@@ -1,24 +1,21 @@
 /*
- * wavemon - a wireless network monitoring aplication
+ * wavemon - a wireless network monitoring application
  *
  * Copyright (c) 2001-2002 Jan Morgenstern <jan@jm-music.de>
- * Copyright (c) 2009      Gerrit Renker <gerrit@erg.abdn.ac.uk>
  *
- * wavemon is free software; you can redistribute it and/or modify it under
- * the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2, or (at your option) any later
- * version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- * wavemon is distributed in the hope that it will be useful, but WITHOUT ANY
- * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
- * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
- * details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with wavemon; see the file COPYING.  If not, write to the Free Software
- * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
-#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -33,9 +30,12 @@
 #include <string.h>
 #include <ctype.h>
 #include <math.h>
+#include <stdint.h>
 #include <stdbool.h>
 #ifdef HAVE_LIBNCURSESW
 #define _XOPEN_SOURCE_EXTENDED
+#endif
+#if defined(HAVE_LIBNCURSESW) && defined(HAVE_NCURSESW_CURSES_H)
 #include <ncursesw/curses.h>
 #else
 #include <ncurses.h>
@@ -56,13 +56,13 @@ enum info_screen_geometry {
 	WH_IFACE    = 2,	/* 'Interface' area at the top */
 	WH_LEVEL    = 9,	/* Level meters */
 	WH_STATS    = 3,	/* WiFi statistics area */
-	WH_INFO_MIN = 7,	/* WiFi information area */
+	WH_INFO     = 8,	/* WiFi information area */
 	WH_NET_MIN  = 3,	/* Network interface information area */
 	WH_NET_MAX  = 5,	/* Network interface information area */
 	WH_MENU	    = 1		/* Menu bar at the bottom */
 };
 #define WH_INFO_SCR_BASE	(WH_IFACE + WH_LEVEL + WH_STATS + WH_MENU)
-#define WH_INFO_SCR_MIN		(WH_INFO_SCR_BASE + WH_INFO_MIN + WH_NET_MIN)
+#define WH_INFO_SCR_MIN		(WH_INFO_SCR_BASE + WH_INFO + WH_NET_MIN)
 
 #define MIN_SCREEN_LINES	WH_INFO_SCR_MIN
 #define MIN_SCREEN_COLS		80
@@ -74,6 +74,10 @@ enum info_screen_geometry {
  */
 #define WAV_WIDTH	(COLS)
 #define WAV_HEIGHT	(LINES-1)
+/*
+ * Maximum length of a menubar key entry
+ */
+#define MAX_MENU_KEY	(MIN_SCREEN_COLS/10)
 /*
  * Maximum lengths/coordinates inside the bordered screen.
  *
@@ -121,6 +125,13 @@ enum scan_sort_order {
 	SO_OPEN_SIG
 };
 
+/** Band filtering (2.4 GHz / 5 GHz / Both) */
+enum scan_filter_band {
+	SCAN_FILTER_BAND_BOTH,
+	SCAN_FILTER_BAND_2G,
+	SCAN_FILTER_BAND_5G
+};
+
 /*
  * Global in-memory representation of current wavemon configuration state
  */
@@ -140,14 +151,16 @@ extern struct wavemon_conf {
 		meter_decay;
 
 	/* Boolean values */
-	int	check_geometry,		/* ensure window is large enough */
+	int	check_geometry,		/* Ensure window is large enough */
 		cisco_mac,		/* Cisco-style MAC addresses */
 		transparent_bg,		/* Use terminal background instead of black */
-		override_bounds,	/* override autodetection */
-		scan_sort_asc;		/* direction of @scan_sort_order */
+		override_bounds,	/* Override autodetection */
+		scan_sort_asc,		/* Direction of @scan_sort_order */
+		scan_hidden_essids;	/* Whether to include hidden SSIDs */
 
 	/* Enumerated values */
 	int	scan_sort_order,	/* channel|signal|open|chan/sig ... */
+		scan_filter_band,	/* 2.4ghz|5ghz|both */
 		lthreshold_action,	/* disabled|beep|flash|beep+flash */
 		hthreshold_action,	/* disabled|beep|flash|beep+flash */
 		startup_scr;		/* info|histogram|aplist */
@@ -250,24 +263,15 @@ extern void waddthreshold(WINDOW *win, int y, float v, float tv,
 			  float minv, float maxv, int8_t *cscale, chtype tch);
 enum colour_pair {
 	CP_STANDARD = 1,
-	CP_SCALEHI,
-	CP_SCALEMID,
-	CP_SCALELOW,
-	CP_WTITLE,
-	CP_INACTIVE,
-	CP_ACTIVE,
-	CP_STATSIG,
-	CP_STATNOISE,
-	CP_STATSNR,
-	CP_STATBKG,
-	CP_STATSIG_S,
-	CP_STATNOISE_S,
-	CP_PREF_NORMAL,
-	CP_PREF_SELECT,
-	CP_PREF_ARROW,
-	CP_SCAN_CRYPT,
-	CP_SCAN_UNENC,
-	CP_SCAN_NON_AP
+	CP_RED,
+	CP_YELLOW,
+	CP_GREEN,
+	CP_CYAN,
+	CP_BLUE,
+	CP_RED_ON_BLUE,
+	CP_GREEN_ON_BLUE,
+	CP_CYAN_ON_BLUE,
+	CP_BLUE_ON_BLUE,
 };
 
 static inline int cp_from_scale(float value, int8_t const *cscale, bool reverse)
@@ -275,11 +279,11 @@ static inline int cp_from_scale(float value, int8_t const *cscale, bool reverse)
 	enum colour_pair cp;
 
 	if (value < (float)cscale[0])
-		cp = reverse ? CP_SCALEHI : CP_SCALELOW;
+		cp = reverse ? CP_RED : CP_GREEN;
 	else if (value < (float)cscale[1])
-		cp = CP_SCALEMID;
+		cp = CP_YELLOW;
 	else
-		cp = reverse ? CP_SCALELOW : CP_SCALEHI;
+		cp = reverse ? CP_GREEN : CP_RED;
 
 	return COLOR_PAIR(cp);
 }
